<?php
/*
Plugin Name: Tippy
Plugin URI: http://croberts.me/tippy/
Description: Simple plugin to display tooltips within your WordPress blog.
Version: 4.2.0
Author: Chris Roberts
Author URI: http://croberts.me/
*/

/*  Copyright 2013 Chris Roberts (email : chris@dailycross.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Add to the Admin function list
if (! function_exists('tippy_addoptions')) {
	function tippy_addoptions()
	{
		if (function_exists('add_options_page')) {
			require_once(plugin_dir_path(__FILE__) .'/tippy_admin.php');
			add_options_page('Tippy Plugin Options', 'Tippy', 'manage_options', basename(__FILE__), 'tippy_options_subpanel');
		}
	}
}

// Receive necessary data and shape it into a Tippy link
/* $tippyArray[
 * 'title' => Optional string (either this or img must be set); should first be passed through tippy_format_title($title).
 * 'swaptitle' => Optional string; text to show when hovering over the Tippy link. Does not work with an image.
 * 'img' => Optional string (either this or title must be set); users may want Tippy to trigger on an image rather than text. This should be the full address to the image.
 * 'text' => Required string; should first be passed through tippy_format_text(do_shortcode($text)).
 * 'href' => Optional string; contains link href for the Tippy link, if desired.
 * 'header' => Optional string (on/off); specifies if the header is on or off.
 * 'headerhref' => Optional string; by default, header link uses the href value; this overrides the link used in the header.
 * 'headertext' => Optional string; by default, the header displays the title value; this specifies the text to show in the header.
 * 'autoclose' => Optional string; by default, uses global setting; allows per-tooltip specification of autoclose.
 * 'class' => Optional string; specify an additional class for the tooltip link. Class gets passed on to the tooltip with _tip appended.
 * 'id' => Optional string; specifies id for the link. Gets passed on to the tooltip with _tip appended.
 * 'item' => Optional string or int; used to automatically create unique identifiers for the tooltips.
 * 'height' => Optional int; manually specify tooltip height in pixels.
 * 'width' => Optional int; manually specify tooltip width in pixels.
 * 'top' => Optional int; specifies top position in pixels.
 * 'left' => Optional int; specifies left position in pixels.
 * 'bottom' => Optional int; specifies bottom position in pixels.
 * 'right' => Optional int; specifies right position in pixels.
 * ];
*/
if (!function_exists('tippy_getLink')) {
    function tippy_getLink($tippyArray)
    {
        // Initialize values that might default empty
        $tippyTarget = '';
        $tippyMouseOut = '';
        $tippyTitleAttribute = '';
        $tippyLinkClass = 'tippy_link';
        
        // Specify default return value. Defaults to the text passed from Tippy.
        // A later check might swap this with the title.
        $returnText = $tippyArray['text'];
        
        // Make sure array values are clean
        $tippyTitle = isset($tippyArray['title']) ? trim($tippyArray['title']) : '';
        $tippySwapTitle = isset($tippyArray['swaptitle']) ? trim($tippyArray['swaptitle']) : '';
        $tippyText = isset($tippyArray['text']) ? trim($tippyArray['text']) : '';
        $tippyHref = isset($tippyArray['href']) ? trim($tippyArray['href']) : false;
        $tippyHeader = isset($tippyArray['header']) ? strtolower(trim($tippyArray['header'])) : false;
        $tippyHeaderHref = isset($tippyArray['headerhref']) ? trim($tippyArray['headerhref']) : false;
        $tippyHeaderText = isset($tippyArray['headertext']) ? trim($tippyArray['headertext']) : false;
        $tippyAutoclose = isset($tippyArray['autoclose']) ? trim($tippyArray['autoclose']) : false;
        $tippyDelay = isset($tippyArray['delay']) ? trim($tippyArray['delay']) : false;
        $tippyClass = isset($tippyArray['class']) ? trim($tippyArray['class']) : false;
        $tippyId = isset($tippyArray['id']) ? trim($tippyArray['id']) : false;
        $tippyItem = isset($tippyArray['item']) ? trim($tippyArray['item']) : false;
        $tippyHeight = isset($tippyArray['height']) ? $tippyArray['height'] : false;
        $tippyWidth = isset($tippyArray['width']) ? $tippyArray['width'] : false;
        $tippyOffsetX = isset($tippyArray['offsetx']) ? $tippyArray['offsetx'] : false;
        $tippyOffsetY = isset($tippyArray['offsety']) ? $tippyArray['offsety'] : false;
        $tippyImg = isset($tippyArray['img']) ? $tippyArray['img'] : false;
        
        $tippyLinkText = '';
        
        // Check our values and build the Tippy parameters
        if (!empty($tippyTitle)) {
            $returnText = $tippyTitle;
        }
        
        // See if the user specified an img
        if (!empty($tippyImg)) {
            $tippyLinkText = '<img src="'. $tippyImg .'"';
            
            if (!empty($tippyTitle)) {
                $tippyLinkText .= ' alt="'. $tippyTitle .'"';
            }
            
            if (!empty($tippyClass)) {
                $tippyLinkText .= ' class="'. $tippyClass .'_img"';
            }
            
            $tippyLinkText .= ' />';
        } else {
            $tippyLinkText = $tippyTitle;
        }
        
        // Check required values: linktext (either title or img) and text
        if (!empty($tippyText) && !empty($tippyLinkText)) {
            // Check href and trigger method
            if (get_option('tippy_openTip', 'hover') == "hover") {
                $activateTippy = "onmouseover";
                
                $tippyHref = trim($tippyHref);
                if (!empty($tippyHref))
                {
                    $tippyHref = 'href="'. $tippyHref .'" ';
                }
            } else {
                $activateTippy = "onmouseup";
            }
            
            // Check the link target
            if (get_option('tippy_linkWindow', 'same') == "new") {
                $tippyTarget = 'target="_blank" ';
            }
            
            // Should Tippy be sticky?
            if ($tippyAutoclose !== 'false' && get_option('tippy_sticky', 'false') == 'false') {
                $tippyMouseOut = 'onmouseout="Tippy.fadeTippyOut();"';
            }
            
            // Should the link use the title attribute?
            if (get_option('tippy_showTitle', 'true') == 'true' && !empty($tippyTitle)) {
                $tippyTitleAttribute = sprintf('title="%s" ', $tippyTitle);
            }
            
            $tippyObject = sprintf("title: '%s', text: '%s'", $tippyTitle, htmlentities($tippyText, ENT_QUOTES, 'UTF-8'));
            
            // See if we have a swap title
            if (!empty($tippySwapTitle)) {
	            $tippyObject .= sprintf(", swaptitle: '%s'", $tippySwapTitle);
            }
            
            // Check the header; allow a variety of possibilities
            if ($tippyHeader === true || $tippyHeader === "on" || $tippyHeader === "yes" || $tippyHeader === "true") {
                $tippyObject .= sprintf(", header: '%s'", $tippyTitle);
                
                // Check the header link
                if (!empty($tippyHeaderHref)) {
                    $tippyObject .= sprintf(", headerhref: '%s'", $tippyHeaderHref);
                }
                
                // Check the header text
                if (!empty($tippyHeaderText)) {
                    $tippyObject .= sprintf(", headerText: '%s'", $tippyHeaderText);
                }
            }
            
            // Check width and height
            if ($tippyWidth !== false) {
                $tippyObject .= sprintf(", width: %d", (int)$tippyWidth);
            }
            
            if ($tippyHeight !== false) {
                $tippyObject .= sprintf(", height: %d", (int)$tippyHeight);
            }
            
            // Check the offsets
            if ($tippyOffsetX !== false) {
                $tippyObject .= sprintf(", offsetx: %d", (int)$tippyOffsetX);
            }
            
            if ($tippyOffsetY !== false) {
                $tippyObject .= sprintf(", offsety: %d", (int)$tippyOffsetY);
            }
            
            // Check class/id
            if (!empty($tippyClass)) {
                $tippyLinkClass .= " ". $tippyClass;
                $tippyObject .= sprintf(", tippyclass: '%s'", $tippyClass);
            }
            
            if (!empty($tippyId)) {
                $tippyObject .= sprintf(", id: '%s'", $tippyId);
            } else {
                $tippyId = 'tippy_tip'. $tippyItem .'_'. rand(100, 9999);
                $tippyObject .= sprintf(", id: '%s'", $tippyId);
            }
            
            // Check delay
            if (!empty($tippyDelay)) {
                $tippyObject .= sprintf(", delay: %d", $tippyDelay);
            }
            
            $returnText = sprintf('<a id="%s" class="%s" %s %s %s %s="Tippy.loadTip({ %s, event: event });" %s>%s</a>', $tippyId, $tippyLinkClass, $tippyHref, $tippyTarget, $tippyTitleAttribute, $activateTippy, $tippyObject, $tippyMouseOut, $tippyLinkText);
        }
        
        return $returnText;
    }
}

// tippy_formatLink() is deprecated; use tippy_getLink() instead
if (!function_exists('tippy_formatLink') ) {
	function tippy_formatLink($tippyShowHeader, $tippyTitle, $tippyHref, $tippyText, $tippyCustomClass, $tippyItem, $tippyWidth = false, $tippyHeight = false)
	{
	    $tippyLink = tippy_getLink(array('header' => $tippyShowHeader, 'title' => $tippyTitle, 'href' => $tippyHref, 'text' => $tippyText, 'class' => $tippyCustomClass, 'item' => $tippyItem, 'width' => $tippyWidth, 'height' => $tippyHeight));
        
		return $tippyLink; 
	}
}

// Pull data out of the shortcode and pass it to format link
if (!function_exists('tippy_shortcode')) {
	// Keep track of how many tooltips showing
	$tippy_tipCount = 0;
	function tippy_shortcode($tippy_attributes, $tippyText = '')
	{
		global $tippy_tipCount;
		
		$link = "";
		
		$tippyAtts = shortcode_atts(array(
            			 'title' => '',
            			 'swaptitle' => '',
            			 'reference' => false, 
            			 'href' => false, 
            			 'header' => 'on', 
            			 'headerhref' => false,
            			 'headertext' => false,
            			 'autoclose' => true,
            			 'delay' => false,
            			 'class' => false, 
            			 'id' => false, 
            			 'height' => false, 
            			 'width' => false,
            			 'offsetx' => false,
            			 'offsety' => false,
            			 'img' => false
        		     ), $tippy_attributes);
		
        if (!empty($tippyAtts['href'])) {
			$link = $tippyAtts['href'];
		} else if (!empty($tippyAtts['reference'])) {
			$link = $tippyAtts['reference'];
		}
		
		$tippyArr = array(
		  'title' => tippy_format_title($tippyAtts['title']),
		  'swaptitle' => tippy_format_title($tippyAtts['swaptitle']),
		  'text' => tippy_format_text(do_shortcode($tippyText)),
		  'href' => $link,
		  'header' => $tippyAtts['header'],
		  'headerhref' => $tippyAtts['headerhref'],
		  'headertext' => tippy_format_title($tippyAtts['headertext']),
		  'autoclose' => $tippyAtts['autoclose'],
		  'delay' => $tippyAtts['delay'],
		  'class' => $tippyAtts['class'],
		  'id' => $tippyAtts['id'], 
		  'item' => $tippy_tipCount,
		  'height' => $tippyAtts['height'],
		  'width' => $tippyAtts['width'],
		  'offsetx' => $tippyAtts['offsetx'],
		  'offsety' => $tippyAtts['offsety'],
		  'img' => $tippyAtts['img']
        );
        
		$tippyLink = tippy_getLink($tippyArr);
		
		$tippy_tipCount++;
		
		return $tippyLink;
	}
}

if (! function_exists('tippy_format_title'))
{
	function tippy_format_title($tippy_title)
	{
		$tippy_title = str_replace("\\", "\\\\", $tippy_title);
		$tippy_title = str_replace("'", "&#8217;", $tippy_title);
		
		return $tippy_title;
	}
}

if (! function_exists('tippy_format_text'))
{
	function tippy_format_text($tippy_text)
	{
		// Just need to replace a few items so do it manually rather than using a function like htmlentities()
		$tippy_text = str_replace("\n", "", $tippy_text);
		$tippy_text = str_replace("\r", "", $tippy_text);
		$tippy_text = str_replace("'", "&#8217;", $tippy_text);
		$tippy_text = str_replace("\\", "&#92;", $tippy_text);
		$tippy_text = str_replace("(", "&#40;", $tippy_text);
		$tippy_text = str_replace(")", "&#41;", $tippy_text);
		
		return $tippy_text;
	}
}

if (! function_exists('tippy_load_scripts')) {
	function tippy_load_scripts()
	{
		// Load jQuery, if not already present
		wp_enqueue_script('jquery');
		
		// Load the Tippy script
		wp_register_script('Tippy', plugins_url() .'/tippy/dom_tooltip.js', array('jquery'), '4.1.0');
		wp_enqueue_script('Tippy');
	}
}

if (! function_exists('tippy_load_styles')) {
	function tippy_load_styles()
	{
		// Load the Tippy css. Checks for several possibilities with the css file.
		if (file_exists(get_stylesheet_directory() .'/dom_tooltip.css')) {
			$tippyCSS = get_bloginfo('stylesheet_directory') .'/dom_tooltip.css';
		} else if (file_exists(get_stylesheet_directory() .'/dom_tooltip.factory.css')) {
			$tippyCSS = get_bloginfo('stylesheet_directory') .'/dom_tooltip.factory.css';
		} else if (file_exists(WP_PLUGIN_DIR .'/tippy/dom_tooltip.css')) {
			$tippyCSS = plugins_url() .'/tippy/dom_tooltip.css';
		}
		
		wp_register_style('Tippy', $tippyCSS);
		wp_enqueue_style('Tippy');
	}
}

if (! function_exists('tippy_load_settings')) {
	function tippy_load_settings()
	{
		if (get_option('tippy_fadeTip', 'fade') == "fade") {
			$tippyFadeRate = 300;
		} else {
			$tippyFadeRate = 0;
		}
		
		echo '
			<script type="text/javascript">
				Tippy.initialize({
					tipPosition: "'. get_option("tippy_tipPosition", "link") .'",
					tipOffsetX: '. get_option("tippy_tipOffsetX", 0) .',
					tipOffsetY: '. get_option("tippy_tipOffsetY", 10) .',
					fadeRate: '. $tippyFadeRate .',
					sticky: '. get_option("tippy_sticky", "false") .',
					showClose: '. get_option("tippy_showClose", "true") .',
					closeText: "'. get_option("tippy_closeLinkText", "Close") .'"
				});
			</script>

		';
	}
}

add_action('admin_menu', 'tippy_addoptions');
add_action('wp_print_scripts', 'tippy_load_scripts');
add_action('wp_enqueue_scripts', 'tippy_load_styles');
add_action('admin_init', 'tippy_load_styles');
add_action('wp_head', 'tippy_load_settings');

add_shortcode('tippy', 'tippy_shortcode');

?>